//
//  CodeView.m
//  VerificationCode
//
//  Created by Crazy凡 on 2016/9/23.
//  Copyright © 2016年 Crazy凡. All rights reserved.
//

#import "CodeView.h"
#import "PuzzleSide.h"
#import "CoverView.h"

#define CODE_IMGAE_SIZE 150


@interface CodeView ()
@property (nonatomic, strong) UIImageView * imageView; 
@property (nonatomic, strong) UIImageView * dropImageView;
@property (nonatomic, strong) CoverView * coverView;
@property (nonatomic, strong) UILabel * messageLabel;

@property (nonatomic, assign) BOOL isMoveAble;
@property (nonatomic, assign) CGPoint defultDropCenter;
@property (nonatomic, assign) CGPoint correctPoint;

@end

@implementation CodeView

- (instancetype)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        [self addSubview:({
            CGRect imageViewFrame = self.bounds;
            imageViewFrame.size.height -= CODE_IMGAE_SIZE;
            _imageView = [[UIImageView alloc] initWithFrame:imageViewFrame];
            _imageView.userInteractionEnabled = NO;
            _imageView;
        })];
        
        [self addSubview:({
            _dropImageView = [[UIImageView alloc] initWithFrame:CGRectMake(0, CGRectGetMaxY(_imageView.frame), CODE_IMGAE_SIZE, CODE_IMGAE_SIZE)];
            [_dropImageView setContentMode:UIViewContentModeCenter];
            _dropImageView.userInteractionEnabled = NO;
            _dropImageView;
        })];
        
        _defultDropCenter = _dropImageView.center;
        
        [self addSubview:({
            _messageLabel = [[UILabel alloc] initWithFrame:CGRectMake(CODE_IMGAE_SIZE,  CGRectGetMaxY(_imageView.frame), CGRectGetWidth(_imageView.frame) - CODE_IMGAE_SIZE, CODE_IMGAE_SIZE)];
            _messageLabel.numberOfLines = 2;
            _messageLabel.text = @"拖动图片到图中阴影位置完成验证";
            _messageLabel.userInteractionEnabled = NO;
            _messageLabel;
        })];
        
        [self addSubview:({
            _coverView = [[CoverView alloc] initWithFrame:_dropImageView.frame];
            _coverView;
        })];
        
        [self bringSubviewToFront:_dropImageView];
        
        _passDistance = 5;
    }
    return self;
}

- (void)setCodeImage:(UIImage*)image
{
    UIImage * needImage = [self cutAndResizeImage:image];
    [_imageView setImage:needImage];
    
    UIBezierPath * path = [self dropImagePath];
    UIImage * dropImage = [self maskImage:[self cutImage:needImage withRect:[self randomRect]] toPath: path];
    
    [_coverView addCoverPath:path];
    _coverView.center = _correctPoint;
    
    [_dropImageView setImage:dropImage];
}

- (UIImage *)cutAndResizeImage:(UIImage *)sourceImage
{
    CGSize needSize = _imageView.frame.size;
    CGSize imageSize = sourceImage.size;
    CGFloat scaleValue;
    if (imageSize.width / needSize.width < imageSize.height / needSize.height) {
        scaleValue = needSize.width / imageSize.width;
    } else {
        scaleValue = needSize.height / imageSize.height;
    }
    CGSize scaleSize = CGSizeMake(needSize.width / scaleValue, needSize.height / scaleValue);
    CGRect cutRect = CGRectMake((imageSize.width - scaleSize.width) / 2, (imageSize.height - scaleSize.height) / 2, 0, 0);
    cutRect.size = scaleSize;
    
    return [self scaleImage:[self cutImage:sourceImage withRect:cutRect] toSize:needSize];
}

- (UIImage *)cutImage:(UIImage *)sourceImage  withRect:(CGRect)rect
{
    CGImageRef imageRef = CGImageCreateWithImageInRect(sourceImage.CGImage, rect);
    UIImage *thumbImage = [UIImage imageWithCGImage:imageRef];
    CGImageRelease(imageRef);
    
    return thumbImage;
}


- (UIImage*)scaleImage:(UIImage *)image toSize:(CGSize)size
{
    UIGraphicsBeginImageContext(size);
    [image drawInRect:CGRectMake(0, 0, size.width, size.height)];
    UIImage* scaledImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    
    return scaledImage;
}

- (UIImage*)maskImage:(UIImage*)originImage toPath:(UIBezierPath*)path
{
    UIGraphicsBeginImageContextWithOptions(originImage.size, NO, 0);
    [path addClip];
    [originImage drawAtPoint:CGPointZero];
    UIImage *img = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return img;
}

- (UIBezierPath *)dropImagePath
{
    CGRect cutRect = [self cutRect:CGRectMake(0, 0, CODE_IMGAE_SIZE, CODE_IMGAE_SIZE)];
    CGPoint points[4];
    for (int i = 0; i < 4; i++) {
        points[i] = cutRect.origin;
    }
    points[1].x += cutRect.size.width;
    points[2].x += cutRect.size.width;
    points[2].y += cutRect.size.height;
    points[3].y += cutRect.size.height;
    
    UIBezierPath * path = [UIBezierPath bezierPath];
    [path moveToPoint:points[0]];
    [[PuzzleSide puzzleSideWithPointA:points[0] andPointB:points[1] andType:[self randomType]] addPointToPath:path];
    [[PuzzleSide puzzleSideWithPointA:points[1] andPointB:points[2] andType:[self randomType]] addPointToPath:path];
    [[PuzzleSide puzzleSideWithPointA:points[2] andPointB:points[3] andType:[self randomType]] addPointToPath:path];
    [[PuzzleSide puzzleSideWithPointA:points[3] andPointB:points[0] andType:[self randomType]] addPointToPath:path];
    [path closePath];
    
    return path;
}

- (HumpType)randomType
{
    CGFloat randomNum = [self customRandom];
    
    return randomNum < 1.0 / 3.0 ? HumpTypeNone : randomNum < 2.0 / 3.0 ? HumpTypeClockwise : HumpTypeCounterClockwise ;
}
                     
- (CGRect)cutRect:(CGRect)rect
{
    CGFloat widthPart = rect.size.width / 5.0;
    CGFloat heightPart = rect.size.height / 5.0;
    rect.origin.x += widthPart;
    rect.origin.y += heightPart;
    rect.size.width -= 2 * widthPart;
    rect.size.height -= 2 * heightPart;
    
    return rect;
}

- (CGRect)randomRect
{
    CGSize canUseSize = _imageView.frame.size;
    canUseSize.width -= CODE_IMGAE_SIZE;
    canUseSize.height -= CODE_IMGAE_SIZE;
    
    CGPoint startPoint = CGPointMake(canUseSize.width * [self customRandom] , canUseSize.height * [self customRandom]);
    CGRect dropImageRect =  CGRectZero;
    dropImageRect.origin = startPoint;
    dropImageRect.size = CGSizeMake(CODE_IMGAE_SIZE, CODE_IMGAE_SIZE);
    
    _correctPoint = dropImageRect.origin;
    _correctPoint.x += CGRectGetWidth(dropImageRect) / 2.0;
    _correctPoint.y += CGRectGetHeight(dropImageRect) / 2.0;
    
    return dropImageRect;
}


- (CGFloat)customRandom{
    return arc4random() % 100 / 100.0;
}

- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event{
    if (CGRectContainsPoint(_dropImageView.frame, [self pointFrom:touches])) {
        _isMoveAble = YES;
    }
}

- (void)touchesMoved:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event{
    if (_isMoveAble) {
        _dropImageView.center = [self pointFrom:touches];
    }
}

- (void)touchesEnded:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event{
    if (_isMoveAble) {
        _dropImageView.center = [self pointFrom:touches];
        if ([self distanceFrom:_dropImageView.center to:_correctPoint] < _passDistance) {
            _dropImageView.center = _correctPoint;
            [_delegate codeViewAuthenticationResult:YES];
        } else {
            _dropImageView.center = _defultDropCenter;
            [_delegate codeViewAuthenticationResult:NO];

        }
    }
}

- (CGFloat)distanceFrom:(CGPoint)pointA to:(CGPoint)pointB{
    return sqrt(pow(pointA.x - pointB.x, 2) + pow(pointA.y - pointB.y, 2));
}

- (CGPoint)pointFrom:(NSSet<UITouch *> *)touches{
    return [[touches anyObject] locationInView:self];
}

/*
 // Only override drawRect: if you perform custom drawing.
 // An empty implementation adversely affects performance during animation.
 - (void)drawRect:(CGRect)rect {
 // Drawing code
 }
*/

@end
