//
//  PuzzleSide.m
//  VerificationCode
//
//  Created by Crazy凡 on 2016/9/23.
//  Copyright © 2016年 Crazy凡. All rights reserved.
//

#import "PuzzleSide.h"

@interface PuzzleSide ()
@property (nonatomic, assign) CGPoint pointA;
@property (nonatomic, assign) CGPoint pointB;

@property (nonatomic, assign) CGPoint linePointA;
@property (nonatomic, assign) CGPoint linePointB;
@property (nonatomic, assign) CGPoint linePointC;

@property (nonatomic, assign) CGPoint controlPointA;
@property (nonatomic, assign) CGPoint controlPointB;
@property (nonatomic, assign) CGPoint controlPointC;
@property (nonatomic, assign) CGPoint controlPointD;

@property (nonatomic, assign) CGFloat lengthX;
@property (nonatomic, assign) CGFloat lengthY;

@property (nonatomic, assign) HumpType type;
@end

@implementation PuzzleSide

+ (instancetype)puzzleSideWithPointA:(CGPoint)pointA andPointB:(CGPoint)pointB andType:(HumpType)type
{
    PuzzleSide *side = [[PuzzleSide alloc] init];
    side.pointA = pointA;
    side.pointB = pointB;
    side.type = type;
    [side makeBezierPathPoint];
    return side;
}

- (void)makeBezierPathPoint{
    _lengthX = _pointB.x - _pointA.x;
    _lengthY = _pointB.y - _pointA.y;
    CGPoint t1, t2;

    t1 = [self percentPointWithPercent:sqrt(5.0) / 6.0];
    t2 = [self percentPointWithPercent:1.0 - sqrt(5.0) / 6.0];
    
    
    _linePointA = [self percentPointWithPercent:1.0 / 3.0];
    _linePointC = [self percentPointWithPercent:2.0 / 3.0];
    
    
    _controlPointA = [self percentPointWithPercent:37.0 / 60.0];
    _controlPointB = [self controlPointWithPoint:t1 andBasePoint:_pointA andDirection:_type];
    _controlPointC = [self controlPointWithPoint:t2 andBasePoint:_pointB andDirection:-_type];
    _controlPointD = [self percentPointWithPercent:23.0 / 60.0];
    
    _linePointB = CGPointMake((_controlPointB.x + _controlPointC.x) / 2, (_controlPointB.y + _controlPointC.y) / 2);
}

- (CGPoint)controlPointWithPoint:(CGPoint)point andBasePoint:(CGPoint)basePoint andDirection:(int)direction
{
    CGPoint tempPoint = CGPointMake(point.x - basePoint.x, point.y - basePoint.y);
    
    CGFloat radian = acosf(sqrt(5.0) / 5.0);
    radian = direction > 0 ? radian : -radian;
    CGFloat sinR = sin(radian);
    CGFloat cosR = cos(radian);
    
    return CGPointMake(tempPoint.x * cosR - tempPoint.y * sinR + basePoint.x, tempPoint.x * sinR + tempPoint.y * cosR + basePoint.y);
}

- (CGPoint)percentPointWithPercent:(CGFloat)percent
{
    return CGPointMake(_pointA.x + (_pointB.x - _pointA.x) * percent, _pointA.y + (_pointB.y - _pointA.y) * percent);
}

- (UIBezierPath *)path
{
    UIBezierPath * path = [UIBezierPath bezierPath];
    return [self addPointToPath:path];
}

- (UIBezierPath *)addPointToPath:(UIBezierPath *)path{
    [path addLineToPoint:_pointA];
    if (_type != HumpTypeNone) {
        [path addLineToPoint:_linePointA];
        [path addCurveToPoint:_linePointB controlPoint1:_controlPointA controlPoint2:_controlPointB];
        [path addCurveToPoint:_linePointC controlPoint1:_controlPointC controlPoint2:_controlPointD];
    }
    [path addLineToPoint:_pointB];
    return path;
}

@end
